<?php
/*************************************************************************************************
 * Copyright 2012 JPL TSolucio, S.L.  --  This file is a part of vtyiiCPNG.
* You can copy, adapt and distribute the work under the "Attribution-NonCommercial-ShareAlike"
* Vizsage Public License (the "License"). You may not use this file except in compliance with the
* License. Roughly speaking, non-commercial users may share and modify this code, but must give credit
* and share improvements. However, for proper details please read the full License, available at
* http://vizsage.com/license/Vizsage-License-BY-NC-SA.html and the handy reference for understanding
* the full license at http://vizsage.com/license/Vizsage-Deed-BY-NC-SA.html. Unless required by
* applicable law or agreed to in writing, any software distributed under the License is distributed
* on an  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and limitations under the
* License terms of Creative Commons Attribution-NonCommercial-ShareAlike 3.0 (the License).
*************************************************************************************************/

include_once 'include/Webservices/VtigerModuleOperation.php';
include_once 'modules/Settings/MailScanner/core/MailAttachmentMIME.php';

class VtigerDocumentOperation extends VtigerModuleOperation {
	protected $tabId;
	protected $isEntity = true;

	public function VtigerDocumentOperation($webserviceObject,$user,$adb,$log){
		parent::__construct($webserviceObject,$user,$adb,$log);
		$this->tabId = $this->meta->getTabId();
	}

	/*
	 * This create function supports a few virtual fields for the attachment and the related entities
	 * so it expects and $element array with the normal Document fields and these additional ones:
	 * 
	 * 'attachment'  this is a base64encoded string contaning the full document to be saved internally,
	 *     this will only be checked if filelocationtype=='I'
	 * 
	 * 'attachment_name'  a string with the name of the attachment
	 * 
	 * 'relations'  this is an array of related entity id's, the id's must be in webservice extended format
	 *     all the indicated entities will be related to the document being created
	 * 
	 */
	public function create($elementType,$element){
		global $adb,$log;                 
		$crmObject = new VtigerCRMObject($elementType, false);		

		if ($element['filelocationtype']=='I' and !empty($element['filename'])) {
                        $element['filesize']=$element['filename']['size'];
                        $attachid=$this->__SaveAttachmentDB($element);
                        $element['filetype']=$element['filename']['type'];
                        $element['filename']=$filename = str_replace(' ', '_',$element['filename']['name']);
		}
		$relations=$element['relations'];
		unset($element['relations']);

		$element = DataTransform::sanitizeForInsert($element,$this->meta);

		$error = $crmObject->create($element);
		if(!$error){
			throw new WebServiceException(WebServiceErrorCode::$DATABASEQUERYERROR,"Database error while performing required operation");
		}

		$id = $crmObject->getObjectId();

		$error = $crmObject->read($id);
		if(!$error){
			throw new WebServiceException(WebServiceErrorCode::$DATABASEQUERYERROR,"Database error while performing required operation");
		}

		if ($element['filelocationtype']=='I' and !empty($attachid)) {
			// Link file attached to document
			$adb->pquery("INSERT INTO vtiger_seattachmentsrel(crmid, attachmentsid) VALUES(?,?)",Array($id, $attachid));
		}
		// Establish relations
		if (!empty($relations) and !is_array($relations))
			$relations = array($relations);
		if (!empty($relations) and is_array($relations)) {
		foreach ($relations as $rel) {
			$ids = vtws_getIdComponents($rel);
			$relid = $ids[1];
			if (!empty($relid))
				$adb->pquery("INSERT INTO vtiger_senotesrel(crmid, notesid) VALUES(?,?)",Array($relid, $id));
		}}

		return DataTransform::filterAndSanitize($crmObject->getFields(),$this->meta);
	}

	public function retrieve($id){
		global $adb;
		$ids = vtws_getIdComponents($id);
		$elemid = $ids[1];
		$doc = parent::retrieve($id);
		// Add relations
		$relsrs=$adb->pquery("SELECT crmid FROM vtiger_senotesrel where notesid=?",Array($elemid));
		$rels=array();
		while ($rl = $adb->fetch_array($relsrs)) {
			$rels[]=$this->vtyiicpng_getWSEntityId(getSalesEntityType($rl['crmid'])).$rl['crmid'];
		}
		$doc['relations']=$rels;
		return $doc;
	}

	/*
	 * This method accepts the same virtual fields that the create method does (see create)
	 * 
	 * It will first eliminate the current related attachement and then relate the new attachment
	 * 
	 * It will first eliminate all the current relations and then establish the new ones being sent in
	 * so ALL relations that are needed must sent in again each time
	 * 
	 */
	public function update($element){
		global $adb;
		$ids = vtws_getIdComponents($element["id"]);
		if ($element['filelocationtype']=='I' and !empty($element['filename'])) {
			$element['filesize']=$element['filename']['size'];
                        $attachid=$this->__SaveAttachmentDB($element);
                        $element['filetype']=$element['filename']['type'];
                        $element['filename']=$filename = str_replace(' ', '_',$element['filename']['name']);
		}
		$relations=$element['relations'];
		unset($element['relations']);

		$element = DataTransform::sanitizeForInsert($element,$this->meta);

		$crmObject = new VtigerCRMObject($this->tabId, true);
		$crmObject->setObjectId($ids[1]);
		$error = $crmObject->update($element);
		if(!$error){
			throw new WebServiceException(WebServiceErrorCode::$DATABASEQUERYERROR,"Database error while performing required operation");
		}

		$id = $crmObject->getObjectId();

		$error = $crmObject->read($id);
		if(!$error){
			throw new WebServiceException(WebServiceErrorCode::$DATABASEQUERYERROR,"Database error while performing required operation");
		}

		if ($element['filelocationtype']=='I' and !empty($attachid)) {
			// Link file attached to document
			$adb->pquery("DELETE from vtiger_seattachmentsrel where crmid=?",Array($id));
			$adb->pquery("INSERT INTO vtiger_seattachmentsrel(crmid, attachmentsid) VALUES(?,?)",Array($id, $attachid));
		}
		// Establish relations
		$adb->pquery("DELETE from vtiger_senotesrel where crmid=?",Array($id));
		foreach ($relations as $rel) {
			$ids = vtws_getIdComponents($rel);
			$relid = $ids[1];
			$adb->pquery("INSERT INTO vtiger_senotesrel(crmid, notesid) VALUES(?,?)",Array($relid, $id));
		}

		return DataTransform::filterAndSanitize($crmObject->getFields(),$this->meta);
	}

	/**
	 * Save the attachment to the database
	 */
	private function __SaveAttachmentDB($element) {
		global $adb;
		$attachid = $adb->getUniqueId('vtiger_crmentity');
		$filename = $element['filename']['name'];
		$description = $filename;
		$date_var = $adb->formatDate(date('YmdHis'), true);
		$usetime = $adb->formatDate($date_var, true);
		$userid = vtws_getIdComponents($element['assigned_user_id']);
		$userid = $userid[1];
		$setype = "Documents Attachment";
		$adb->pquery("INSERT INTO vtiger_crmentity(crmid, smcreatorid, smownerid,
				modifiedby, setype, description, createdtime, modifiedtime, presence, deleted)
				VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
				Array($attachid, $userid, $userid, $userid, $setype, $description, $usetime, $usetime, 1, 0));
		$this->__SaveAttachmentFile($attachid, $filename, $element['filename']['content']);
		return $attachid;
	}

	/**
	 * Save the attachment to the file
	 */
	private function __SaveAttachmentFile($attachid, $filename, $filecontent) {
		global $adb,$log;

		$dirname = decideFilePath();
		if(!is_dir($dirname)) mkdir($dirname);

		$description = $filename;
		$filename = str_replace(' ', '_', $filename);
		$saveasfile = "$dirname$attachid" . "_$filename";              
		if(!file_exists($saveasfile)) {                 
			$fh = fopen($saveasfile, 'wb');
			fwrite($fh, base64_decode($filecontent));
			fclose($fh);
		}

		$mimetype = MailAttachmentMIME::detect($saveasfile);
	
		$adb->pquery("INSERT INTO vtiger_attachments SET attachmentsid=?, name=?, description=?, type=?, path=?",
				Array($attachid, $filename, $description, $mimetype, $dirname));
	}

	private function vtyiicpng_getWSEntityId($entityName) {
		global $adb;
		return $adb->getone("select id from vtiger_ws_entity where name='$entityName'").'x';
	}

}
?>
